package edu.pacificu.cs.cs360.PingPong;

import java.nio.ByteBuffer;

public class PingPongConnection {
	
	/* What state is the connection currently in? */
	public static enum State {
		PING, PONG, NONE
	};

	/* Do we need to write a message to the client */
	private boolean needToWrite;

	/*
	 * are we currently running in the handler? do we want the server to ignore
	 * this connection until we finish our read/write work?
	 */
	private boolean running;

	private State currState;

	/*
	 * Reads and writes may take more than one invocation to read/write all the
	 * data we need. So we keep track of the data we have received so far as a
	 * String and the data we are trying to write as a ByteBuffer. Repeated
	 * invocations of socketchannel.write(writeBuffer) update the pointer in the
	 * ByteBuffer and correctly write all the data.
	 * 
	 * Repeated invocations of socketchannel.read(byteBuffer) need to be handled
	 * carefully and the ByteBuffer data transferred to the provided String.
	 */

	/* the data we are writing to the client */
	private ByteBuffer writeBuffer;

	/* the data we are reading from the client */
	private String readString;

	/* constant Strings in the Protocol */
	public static final String PING = "PING";
	public static final String PONG = "PONG";

	/**
	 * Instantiates a new ping pong connection.
	 */
	public PingPongConnection() {
		running = false;
		currState = State.NONE;
		needToWrite = false;
		writeBuffer = ByteBuffer.allocate(1024);
		readString = "";
	}

	/**
	 * Gets the read string.
	 * 
	 * @return the read string
	 */
	public String getReadString() {
		return readString;
	}

	/**
	 * Sets the read string.
	 * 
	 * @param readString
	 *            the new read string
	 */
	public void setReadString(String readString) {
		this.readString = readString;
	}

	/**
	 * Are we running?
	 * 
	 * @return true, if is running
	 */
	public boolean isRunning() {
		return running;
	}

	/**
	 * Sets the running.
	 * 
	 * @param running
	 *            the new running
	 */
	public void setRunning(boolean running) {
		this.running = running;
	}

	/**
	 * Is the inputline a valid string in the protocol?
	 * 
	 * @param inputString
	 *            the input string
	 * @return true, if successful
	 */
	public boolean lineIsValid(String inputString) {
		inputString = inputString.trim();
		inputString = inputString.toUpperCase();

		return inputString.equals(PING) || inputString.equals(PONG);
	}

	/**
	 * Sets the state.
	 * 
	 * @param inputString
	 *            the input string
	 * @return the state
	 */
	public State setState(String inputString) {
		inputString = inputString.trim();
		inputString = inputString.toUpperCase();

		if (inputString.equals(PING)) {
			currState = State.PONG;
		}
		if (inputString.equals(PONG)) {
			currState = State.PING;
		}

		return currState;
	}

	/**
	 * Gets the state.
	 * 
	 * @return the state
	 */
	public State getState() {
		return currState;
	}

	/**
	 * Gets the write buffer.
	 * 
	 * @return the write buffer
	 */
	public ByteBuffer getWriteBuffer() {
		return writeBuffer;
	}

	/**
	 * Checks if is need to write.
	 * 
	 * @return true, if is need to write
	 */
	public boolean isNeedToWrite() {
		return needToWrite;
	}

	/**
	 * Sets the need to write and sets the writeBuffer to the correct data that
	 * needs to be written to the connection
	 * 
	 * @param needToWrite
	 *            the new need to write
	 */
	public void setNeedToWrite(boolean needToWrite) {
		this.needToWrite = needToWrite;
		writeBuffer.clear();
		if (State.PING == currState) {
			writeBuffer.put("*PING\n".getBytes());
		}
		if (State.PONG == currState) {
			writeBuffer.put("*PONG\n".getBytes());
		}
		writeBuffer.flip();
	}

	/**
	 * Checks if the inputLine is a valid prefix of one of the strings defined
	 * in the protocol
	 * 
	 * @param inputLine
	 *            the input line
	 * @return true, if is valid prefix
	 */
	public boolean isValidPrefix(String inputLine) {

		inputLine = inputLine.trim();

		if (PING.startsWith(inputLine) || PONG.startsWith(inputLine)) {
			return true;
		}

		return false;
	}

}
